@@ -51,6 +51,18 @@ test_that("basic usage", {
   expect_warning(prep(rec1), NA)
 })
 
+test_that("bake method errors when needed non-standard role columns are missing", {
+  rec <- recipe(class ~ x + y, data = circle_example) %>%
+    step_smotenc(class, skip = FALSE) %>%
+    add_role(class, new_role = "potato") %>%
+    update_role_requirements(role = "potato", bake = FALSE)
+
+  trained <- prep(rec, training = circle_example, verbose = FALSE)
+
+  expect_error(bake(trained, new_data = circle_example[, -3]),
+               class = "new_data_missing_column")
+})
+
 test_that("printing", {
   rec <- recipe(class ~ x + y, data = circle_example) %>%
     step_smotenc(class)
